/**
 * Copyright (c) 2017-2018 INCHRON GmbH
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     INCHRON GmbH - Meta Model design and implementation
 */
package com.inchron.realtime.root.model.autosar.util;

import com.inchron.realtime.root.Referrable;

import com.inchron.realtime.root.model.CallSequenceItem;
import com.inchron.realtime.root.model.ClockUser;
import com.inchron.realtime.root.model.Component;
import com.inchron.realtime.root.model.Event;
import com.inchron.realtime.root.model.Function;
import com.inchron.realtime.root.model.ModelObject;
import com.inchron.realtime.root.model.Schedulable;
import com.inchron.realtime.root.model.Spinlock;
import com.inchron.realtime.root.model.TraceObject;

import com.inchron.realtime.root.model.autosar.ARSystem;
import com.inchron.realtime.root.model.autosar.AutosarPackage;
import com.inchron.realtime.root.model.autosar.ClearEvent;
import com.inchron.realtime.root.model.autosar.Operation;
import com.inchron.realtime.root.model.autosar.OsAlarm;
import com.inchron.realtime.root.model.autosar.OsAlarmAction;
import com.inchron.realtime.root.model.autosar.OsApplication;
import com.inchron.realtime.root.model.autosar.OsConfig;
import com.inchron.realtime.root.model.autosar.OsCounter;
import com.inchron.realtime.root.model.autosar.OsEvent;
import com.inchron.realtime.root.model.autosar.OsExpiryPoint;
import com.inchron.realtime.root.model.autosar.OsIsr;
import com.inchron.realtime.root.model.autosar.OsResource;
import com.inchron.realtime.root.model.autosar.OsScheduleTable;
import com.inchron.realtime.root.model.autosar.OsSpinlock;
import com.inchron.realtime.root.model.autosar.OsTask;
import com.inchron.realtime.root.model.autosar.ResumeOSInterrupts;
import com.inchron.realtime.root.model.autosar.ServerCall;
import com.inchron.realtime.root.model.autosar.SuspendOSInterrupts;
import com.inchron.realtime.root.model.autosar.SwComponent;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;

import org.eclipse.emf.ecore.util.Switch;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see com.inchron.realtime.root.model.autosar.AutosarPackage
 * @generated
 */
public class AutosarSwitch<T> extends Switch<T> {
	/**
	 * The cached model package
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static AutosarPackage modelPackage;

	/**
	 * Creates an instance of the switch.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public AutosarSwitch() {
		if (modelPackage == null) {
			modelPackage = AutosarPackage.eINSTANCE;
		}
	}

	/**
	 * Checks whether this is a switch for the given package.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param ePackage the package in question.
	 * @return whether this is a switch for the given package.
	 * @generated
	 */
	@Override
	protected boolean isSwitchFor(EPackage ePackage) {
		return ePackage == modelPackage;
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	@Override
	protected T doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case AutosarPackage.AR_SYSTEM: {
				ARSystem arSystem = (ARSystem)theEObject;
				T result = caseARSystem(arSystem);
				if (result == null) result = caseSystem(arSystem);
				if (result == null) result = caseModelObject(arSystem);
				if (result == null) result = caseReferrable(arSystem);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.CLEAR_EVENT: {
				ClearEvent clearEvent = (ClearEvent)theEObject;
				T result = caseClearEvent(clearEvent);
				if (result == null) result = caseCallSequenceItem(clearEvent);
				if (result == null) result = caseModelObject(clearEvent);
				if (result == null) result = caseReferrable(clearEvent);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OPERATION: {
				Operation operation = (Operation)theEObject;
				T result = caseOperation(operation);
				if (result == null) result = caseModelObject(operation);
				if (result == null) result = caseReferrable(operation);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_ALARM: {
				OsAlarm osAlarm = (OsAlarm)theEObject;
				T result = caseOsAlarm(osAlarm);
				if (result == null) result = caseModelObject(osAlarm);
				if (result == null) result = caseReferrable(osAlarm);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_ALARM_ACTION: {
				OsAlarmAction osAlarmAction = (OsAlarmAction)theEObject;
				T result = caseOsAlarmAction(osAlarmAction);
				if (result == null) result = caseModelObject(osAlarmAction);
				if (result == null) result = caseReferrable(osAlarmAction);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_APPLICATION: {
				OsApplication osApplication = (OsApplication)theEObject;
				T result = caseOsApplication(osApplication);
				if (result == null) result = caseModelObject(osApplication);
				if (result == null) result = caseReferrable(osApplication);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_CONFIG: {
				OsConfig osConfig = (OsConfig)theEObject;
				T result = caseOsConfig(osConfig);
				if (result == null) result = caseModelObject(osConfig);
				if (result == null) result = caseReferrable(osConfig);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_COUNTER: {
				OsCounter osCounter = (OsCounter)theEObject;
				T result = caseOsCounter(osCounter);
				if (result == null) result = caseModelObject(osCounter);
				if (result == null) result = caseClockUser(osCounter);
				if (result == null) result = caseReferrable(osCounter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_EVENT: {
				OsEvent osEvent = (OsEvent)theEObject;
				T result = caseOsEvent(osEvent);
				if (result == null) result = caseEvent(osEvent);
				if (result == null) result = caseModelObject(osEvent);
				if (result == null) result = caseReferrable(osEvent);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_EXPIRY_POINT: {
				OsExpiryPoint osExpiryPoint = (OsExpiryPoint)theEObject;
				T result = caseOsExpiryPoint(osExpiryPoint);
				if (result == null) result = caseModelObject(osExpiryPoint);
				if (result == null) result = caseReferrable(osExpiryPoint);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_ISR: {
				OsIsr osIsr = (OsIsr)theEObject;
				T result = caseOsIsr(osIsr);
				if (result == null) result = caseProcess(osIsr);
				if (result == null) result = caseSchedulable(osIsr);
				if (result == null) result = caseModelObject(osIsr);
				if (result == null) result = caseTraceObject(osIsr);
				if (result == null) result = caseReferrable(osIsr);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_RESOURCE: {
				OsResource osResource = (OsResource)theEObject;
				T result = caseOsResource(osResource);
				if (result == null) result = caseModelObject(osResource);
				if (result == null) result = caseReferrable(osResource);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_SCHEDULE_TABLE: {
				OsScheduleTable osScheduleTable = (OsScheduleTable)theEObject;
				T result = caseOsScheduleTable(osScheduleTable);
				if (result == null) result = caseModelObject(osScheduleTable);
				if (result == null) result = caseReferrable(osScheduleTable);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_SPINLOCK: {
				OsSpinlock osSpinlock = (OsSpinlock)theEObject;
				T result = caseOsSpinlock(osSpinlock);
				if (result == null) result = caseSpinlock(osSpinlock);
				if (result == null) result = caseModelObject(osSpinlock);
				if (result == null) result = caseTraceObject(osSpinlock);
				if (result == null) result = caseReferrable(osSpinlock);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.OS_TASK: {
				OsTask osTask = (OsTask)theEObject;
				T result = caseOsTask(osTask);
				if (result == null) result = caseProcess(osTask);
				if (result == null) result = caseSchedulable(osTask);
				if (result == null) result = caseModelObject(osTask);
				if (result == null) result = caseTraceObject(osTask);
				if (result == null) result = caseReferrable(osTask);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.RESUME_OS_INTERRUPTS: {
				ResumeOSInterrupts resumeOSInterrupts = (ResumeOSInterrupts)theEObject;
				T result = caseResumeOSInterrupts(resumeOSInterrupts);
				if (result == null) result = caseCallSequenceItem(resumeOSInterrupts);
				if (result == null) result = caseModelObject(resumeOSInterrupts);
				if (result == null) result = caseReferrable(resumeOSInterrupts);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.RUNNABLE: {
				com.inchron.realtime.root.model.autosar.Runnable runnable = (com.inchron.realtime.root.model.autosar.Runnable)theEObject;
				T result = caseRunnable(runnable);
				if (result == null) result = caseFunction(runnable);
				if (result == null) result = caseModelObject(runnable);
				if (result == null) result = caseTraceObject(runnable);
				if (result == null) result = caseReferrable(runnable);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.SERVER_CALL: {
				ServerCall serverCall = (ServerCall)theEObject;
				T result = caseServerCall(serverCall);
				if (result == null) result = caseCallSequenceItem(serverCall);
				if (result == null) result = caseModelObject(serverCall);
				if (result == null) result = caseReferrable(serverCall);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.SUSPEND_OS_INTERRUPTS: {
				SuspendOSInterrupts suspendOSInterrupts = (SuspendOSInterrupts)theEObject;
				T result = caseSuspendOSInterrupts(suspendOSInterrupts);
				if (result == null) result = caseCallSequenceItem(suspendOSInterrupts);
				if (result == null) result = caseModelObject(suspendOSInterrupts);
				if (result == null) result = caseReferrable(suspendOSInterrupts);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case AutosarPackage.SW_COMPONENT: {
				SwComponent swComponent = (SwComponent)theEObject;
				T result = caseSwComponent(swComponent);
				if (result == null) result = caseComponent(swComponent);
				if (result == null) result = caseModelObject(swComponent);
				if (result == null) result = caseReferrable(swComponent);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			default: return defaultCase(theEObject);
		}
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>AR System</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>AR System</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseARSystem(ARSystem object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clear Event</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clear Event</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClearEvent(ClearEvent object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Operation</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Operation</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOperation(Operation object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Alarm</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Alarm</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsAlarm(OsAlarm object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Alarm Action</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Alarm Action</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsAlarmAction(OsAlarmAction object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Application</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Application</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsApplication(OsApplication object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Config</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Config</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsConfig(OsConfig object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Counter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Counter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsCounter(OsCounter object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Event</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Event</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsEvent(OsEvent object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Expiry Point</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Expiry Point</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsExpiryPoint(OsExpiryPoint object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Isr</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Isr</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsIsr(OsIsr object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Resource</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Resource</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsResource(OsResource object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Schedule Table</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Schedule Table</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsScheduleTable(OsScheduleTable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Spinlock</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Spinlock</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsSpinlock(OsSpinlock object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Os Task</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Os Task</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseOsTask(OsTask object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Resume OS Interrupts</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Resume OS Interrupts</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseResumeOSInterrupts(ResumeOSInterrupts object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Runnable</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Runnable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseRunnable(com.inchron.realtime.root.model.autosar.Runnable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Server Call</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Server Call</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseServerCall(ServerCall object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Suspend OS Interrupts</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Suspend OS Interrupts</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSuspendOSInterrupts(SuspendOSInterrupts object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Sw Component</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Sw Component</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSwComponent(SwComponent object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Referrable</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Referrable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseReferrable(Referrable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Object</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Object</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseModelObject(ModelObject object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>System</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>System</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSystem(com.inchron.realtime.root.model.System object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Call Sequence Item</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Call Sequence Item</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseCallSequenceItem(CallSequenceItem object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Clock User</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Clock User</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseClockUser(ClockUser object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Event</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Event</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseEvent(Event object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Trace Object</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Trace Object</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseTraceObject(TraceObject object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Schedulable</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Schedulable</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSchedulable(Schedulable object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Process</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Process</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseProcess(com.inchron.realtime.root.model.Process object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Spinlock</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Spinlock</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseSpinlock(Spinlock object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Function</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Function</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseFunction(Function object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>Component</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>Component</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public T caseComponent(Component object) {
		return null;
	}

	/**
	 * Returns the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch, but this is the last case anyway.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpreting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	@Override
	public T defaultCase(EObject object) {
		return null;
	}

} //AutosarSwitch
