/*******************************************************************************
 * Copyright (c) 2018 Robert Bosch GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 *******************************************************************************/

package org.eclipse.app4mc.emfutils.content.extractor.builders

import java.util.ArrayList
import java.util.List
import org.eclipse.emf.common.util.BasicEMap
import org.eclipse.emf.common.util.EList
import org.eclipse.emf.common.util.EMap
import org.eclipse.emf.ecore.EClass
import org.eclipse.emf.ecore.EClassifier
import org.eclipse.emf.ecore.EObject
import org.eclipse.emf.ecore.EPackage
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.emf.ecore.resource.ResourceSet
import org.eclipse.emf.ecore.util.EcoreUtil

class EClassHierarchyBufferBuilder2 {

	var EMap<String, Object> id2ObjectsMap = new BasicEMap

	def public setId2ObjectsMap(EMap<String, Object> id2ObjectsMap) {
		this.id2ObjectsMap = id2ObjectsMap
	}

	def buildBuffer(EClass selectedObjClass) '''
		
		id2ObjectsMap = new BasicEMap
		
		@startuml
		
		 Generation of EClass basic information 
		
		addClassInfo(selectedObjClass)
		
		 Generation of Hierarchy for Super types 

		IF selectedObjClass.ESuperTypes.size > 0
			
			FOR superClass: selectedObjClass.ESuperTypes
				
					 UUID - which will be used for elements lookup 
					var String uuid=EcoreUtil.generateUUID
							
							IF superClass.abstract && superClass.interface
								interface superClass.name "[[uuid]]"
							ELSEIF superClass.abstract
								abstract superClass.name "[[uuid]]"
							ELSEIF superClass.interface
								interface superClass.name "[[uuid]]"
							ELSE
								class  superClass.name "[[uuid]]"
								
							ENDIF
							
				
				id2ObjectsMap.put(uuid, superClass)
				
				superClass.name <|--selectedObjClass.name
				
				
				
			ENDFOR
			
			
		ENDIF
		
		var List<EClass> subClasses=getSubClasses(selectedObjClass)
		
				IF subClasses.size>0
					
				ENDIF
		
		@enduml
		
		
		
	'''

	/**
	 * This method is used to add EClass specific information
	 */
	protected def CharSequence addClassInfo(EClass selectedObjClass) '''IF selectedObjClass.abstract && selectedObjClass.interface
		
			interface selectedObjClass.name
		ELSEIF selectedObjClass.abstract
			abstract selectedObjClass.name
		ELSEIF selectedObjClass.interface
			interface selectedObjClass.name
		ELSE
			class selectedObjClass.name
		ENDIF'''

	/**
	 * This method is used to get the List of sub-classes of a specific EClass
	 */
	def private List<EClass> getSubClasses(EClass eClass) {

		var List<EClass> allSubClasses = new ArrayList

		var ResourceSet resourceSet = eClass.eResource().getResourceSet();

		if (resourceSet != null) {

			// With this approach check is made to find the Sub-Classes for the supplied EClass in all the packages
			// belonging to the resourceset
			var EList<Resource> resources = resourceSet.getResources();

			for (Resource resource : resources) {

				var EList<EObject> contents = resource.getContents();

				if (contents.size() > 0) {
//					var EObject eObject = contents.get(0);
					for (element : contents) {
						if (element instanceof EPackage) {
							allSubClasses.addAll(getSubClasses(eClass, element));
						}

					}
				}

			}
		} else {

			var EPackage ePackage = eClass.getEPackage();
			return getSubClasses(eClass, ePackage);
		}

		return allSubClasses;

	}

	/**
	 * This method is used to get the List of sub-classes of a specific EClass in the scope of EPackage
	 */
	def List<EClass> getSubClasses(EClass selectedObjClass, EPackage ePackage) {

		var List<EClass> allSubClasses = new ArrayList<EClass>();

		var EList<EClassifier> eClassifiers = ePackage.getEClassifiers();

		for (EClassifier eClassifier : eClassifiers) {

			if (eClassifier instanceof EClass) {

				if (!(selectedObjClass == eClassifier) && selectedObjClass.isSuperTypeOf(eClassifier)) {
					allSubClasses.add(eClassifier);
				}

			}
		}
		return allSubClasses;
	}

}
